/*
 * search.c - C source for GNU CHESS
 *
 * Revision: 1991-04-15
 *
 * Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
 * Copyright (c) 1988, 1989, 1990  John Stanback
 *
 * This file is part of CHESS.
 *
 * CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY.  No author or distributor accepts responsibility to anyone for
 * the consequences of using it or for whether it serves any particular
 * purpose or works at all, unless he says so in writing.  Refer to the CHESS
 * General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute CHESS, but
 * only under the conditions described in the CHESS General Public License. A
 * copy of this license is supposed to have been given to you along with
 * CHESS so you can know your rights and responsibilities.  It should be in a
 * file named COPYING.  Among other things, the copyright notice and this
 * notice must be preserved on all copies.
 */
#include "schah.h"
static short int DepthBeyond;
static unsigned short int PrVar[MAXDEPTH];
static unsigned short int TPrVar1, TPrVar2;
extern char mvstr[4][6];
#ifdef DEBUG
unsigned short DBLINE[MAXDEPTH];
struct leaf *dbptr;
#endif

#include "ataks.h"

/* ............    MOVE GENERATION & SEARCH ROUTINES    .............. */
inline
void
repetition (short int *cnt)

/*
 * Check for draw by threefold repetition.
 */

{
  register short i, c;
  register unsigned short m;
  short b[64];

  *cnt = c = 0;
  /* try to avoid work */
  if (GameCnt > Game50 + 2)
    {
#ifdef NOMEMSET
      for (i = 0; i < 64; b[i++] = 0) ;
#else
      memset ((char *) b, 0, sizeof (b));
#endif /* NOMEMSET */
      for (i = GameCnt; i >= Game50; i--)
	{
	  m = GameList[i].gmove;
	  /* does piece exist on diff board? */
	  if (b[m & 0xff])
	    {
	      /* does diffs cancel out, piece back? */
	      if ((b[m >> 8] += b[m & 0xff]) == 0)
		--c;
	      b[m & 0xff] = 0;
	    }
	  else
	    {
	      /* create diff */
	      ++c;
	      /* does diff cancel out another diff? */
	      if (!(b[m >> 8] -= (b[m & 0xff] = board[m & 0xff] +
				  (color[m & 0xff] << 8))))
		--c;;
	    }
	  /* if diff count is 0 we have a repetition */
	  if (c == 0)
	    (*cnt)++;
	}
    }
}

int Zscore;
void
pick (short int p1, short int p2)

/*
 * Find the best move in the tree between indexes p1 and p2. Swap the best
 * move into the p1 element.
 */

{
  register short p, s;
  register short p0, s0;
  struct leaf temp;

  s0 = Tree[p1].score;
  p0 = p1;
  for (p = p1 + 1; p <= p2; p++)
    if ((s = Tree[p].score) > s0)
      {
	s0 = s;
	p0 = p;
      }
  if (p0 != p1)
    {
      temp = Tree[p1];
      Tree[p1] = Tree[p0];
      Tree[p0] = temp;
    }
}

static int TCcount, TCleft;
void
SelectMove (short int side, short int iop)


/*
 * Select a move by calling function search() at progressively deeper ply
 * until time is up or a mate or draw is reached. An alpha-beta window of
 * -Awindow to +Bwindow points is set around the score returned from the
 * previous iteration. If Sdepth != 0 then the program has correctly
 * predicted the opponents move and the search will start at a depth of
 * Sdepth+1 rather than a depth of 1.
 */

{
  static short i, tempb, tempc, tempsf, tempst, xside, rpt;
  static short alpha, beta, score;
  static struct GameRec *g;

  flag.timeout = false;
  xside = otherside[side];
  /* if background mode set to infinite */
  if (iop == 2)
    ResponseTime = 99999;
  else if (TCflag)
    {
      if (TimeControl.moves[side]<1) TimeControl.moves[side]=1;
      /* calculate avg time per move remaining */
      ResponseTime = (TimeControl.clock[side] - 500)
	/ ((2 * (TimeControl.moves[side] ))) - OperatorTime;
      TCleft = (TimeControl.clock[side] - (ResponseTime * TimeControl.moves[side])) / 4;
      if (TCleft > ResponseTime)
	TCleft = ResponseTime;
      TCcount = 0;
      if (ResponseTime < 100)
	ResponseTime = 100;
    }
  else
    /* if no time control set level */
    ResponseTime = Level;
  
  ExtraTime = 0;
  ExaminePosition ();
  ScorePosition (side, &score);
  ShowSidetoMove ();
  if (TCflag)
    if (score < SCORETIME)
      ResponseTime += TCleft;

  SearchStartStuff (side);
  FROMsquare = TOsquare = -1;
  PV = 0;
  if (iop != 2)
    hint = 0;
  for (i = 0; i < MAXDEPTH; i++)
    PrVar[i] = killr0[i] = killr1[i] = killr2[i] = killr3[i] = 0;
  /* set initial window for search */
  alpha = score - ((computer == black) ? BAwindow : WAwindow);
  beta = score + ((computer == black) ? BBwindow : WBwindow);
  rpt = 0;
  TrPnt[1] = 0;
  root = &Tree[0];
  MoveList (side, 1);
  for (i = TrPnt[1]; i < TrPnt[2]; i++)
    pick (i, TrPnt[2] - 1);
  /* can I get a book move? */
  if (flag.regularstart && Book != 0)
    flag.timeout = OpeningBook (&hint, side);
  /* got one so end search */
  /* zero stats for hash table */
  reminus = replus = 0;
  NodeCnt = ETnodes = EvalNodes = HashCnt = FHashAdd = HashAdd = FHashCnt = HashCol = 0;
  DepthBeyond = DEPTHBEYONDRIGHT;
  Zscore = score;

  while (!flag.timeout && Sdepth < MaxSearchDepth)
    {
      /* go down a level at a time */
      Sdepth++;
#if !defined CHESSTOOL && !defined XBOARD
      ShowDepth (' ');
#endif
      /* search at this level returns score of PV */
      score = search (side, 1, Sdepth, alpha, beta, PrVar, &rpt);
      /* save PV as killer */
      for (i = 1; i <= Sdepth; i++)
	killr0[i] = PrVar[i];
      /* low search failure re-search with (-inf,score) limits  */
      if (score < alpha)
	{
	  reminus++;
	  ExtraTime += TCleft;
	  ShowDepth ('-');
	  score = search (side, 1, Sdepth, -9000, score, PrVar, &rpt);
	}
      /* high search failure re-search with (score, +inf) limits */
      if (score > beta && !(root->flags & exact))
	{
	  replus++;
	  ExtraTime += TCleft;
	  ShowDepth ('+');
	  score = search (side, 1, Sdepth, score, 9000, PrVar, &rpt);
	}
      if (TCflag && Sdepth > ZDEPTH && abs (Zscore - score) > ZDELTA)
	{
	  ExtraTime += TCleft;
	  TCcount++;
	}
      Zscore = score;
      TPrVar1 = PrVar[1];
      TPrVar2 = PrVar[2];
      /* find the next best move put below root */
      if (!flag.timeout)
       for (i = TrPnt[1] + 1; i < TrPnt[2]; i++)
	pick (i, TrPnt[2] - 1);
      score = root->score;
      ElapsedTime (2);
#if !defined CHESSTOOL && !defined XBOARD
      ShowResults (score, PrVar, '.');
#endif
#ifdef DEBUG4
if (debuglevel & 4){ int j; for(j=1; j<=Sdepth; j++){
	int idb; for (idb = TrPnt[j]; idb < TrPnt[j+1]; idb++) {
	algbr (Tree[idb].f, Tree[idb].t, 0);printf ("%d-->%d %s %d %d\n", j,idb, mvstr, Tree[idb].score, Tree[idb].width); } }
	}
#endif

      /* save PV as killer */
      for (i = 1; i <= Sdepth; i++)
	killr0[i] = PrVar[i];
      /* if done or nothing good to look at quit */
      if ((root->flags & exact) || (score < -9000))
	flag.timeout = true;
      /* */
      if (!flag.timeout)
	{
	  Tscore[0] = score;
	}
      /* recompute search window */
      alpha = score - ((computer == black) ? BAwindow : WAwindow);
      beta = score + ((computer == black) ? BBwindow : WBwindow);

    }

  if (rpt >= 2)
    {
      root->flags |= draw;
      DRAW = CP[101];		/*Repetition*/
    }
  if (score < -12000)
    {
      root->flags |= draw;
      DRAW = CP[105];		/*Score*/
    }
  if (!PieceCnt[black] && !PieceCnt[white])
    {
      root->flags |= draw;
      DRAW = CP[88];		/*No pieces*/
    }
  /* if no moves and not in check then draw */
  if ((root->score == -9999) && !(SqAtakd (PieceList[side][0], xside)))
    {
      root->flags |= draw;
      DRAW = CP[87];		/*No moves*/
    }
  if (GameCnt == MAXMOVES)
    {
      root->flags |= draw;
      DRAW = CP[80];		/*Max Moves*/
    }
  /* background mode */
  if (iop == 2)
    return;
  /* not in book so set hint to guessed move for other side */
  if (Book == 0)
    hint = PrVar[2];

  /* if not mate or draw make move and output it */
  if (score > -9999 && rpt <= 2 || (root->flags & draw))
    {
      MakeMove (side, root, &tempb, &tempc, &tempsf, &tempst, &INCscore);
      algbr (root->f, root->t, (short) root->flags);
    }
  else
    algbr (0, 0, 0);
  /* If Time Control get the elapsed time */
  if (TCflag)
    ElapsedTime (1);
  OutputMove ();
  /* if mate set flag */
  if (score == -9999 || score == 9998)
    flag.mate = true;
  /* if mate clear hint */
  if (flag.mate)
    hint = 0;
  /* if pawn move or capture or castle or promote zero repitition array */
  if ((board[root->t] == pawn) || (root->flags & (capture | cstlmask | promote)))
    {
      Game50 = GameCnt;
      ZeroRPT ();
    }
  /* add move to game list */
  g = &GameList[GameCnt];
  g->score = score;
  g->nodes = NodeCnt;
  g->time = (short) et;
  g->depth = Sdepth;
  g->flags = root->flags;
  /* update time comtrol info */
  if (TCflag)
    {
      TimeControl.clock[side] -= (et + OperatorTime);
      /* finished our required moves - setup the next set */
      if (--TimeControl.moves[side] == 0)
	SetTimeControl ();
    }
  /* check for end conditions */
  if ((root->flags & draw) && flag.bothsides)
    flag.mate = true;
  else if (GameCnt == MAXMOVES)
    {
      flag.quit = flag.mate = true;
    }
  /* out of move store, you loose */
  else
    /* switch to other side */
    player = xside;
  Sdepth = 0;
}

int
search (short int side,
	register short int ply,
	register short int depth,
	short int alpha,
	short int beta,
	short unsigned int *bstline,
	short int *rpt)

/*
 * Perform an alpha-beta search to determine the score for the current board
 * position. If depth <= 0 only capturing moves, pawn promotions and
 * responses to check are generated and searched, otherwise all moves are
 * processed. The search depth is modified for check evasions, certain
 * re-captures and threats. Extensions may continue for up to 11 ply beyond
 * the nominal search depth.
 */


{
  register short j, pnt;
  short tempb, tempc, tempsf, tempst;
  short xside, pbst, score, rcnt, slk, InChk;
  unsigned short mv, nxtline[MAXDEPTH];
  struct leaf *node, tmp;
  short best;
  short bestwidth = 0;
  NodeCnt++;
  /* look every ZNODE nodes for a timeout */
  if (TCflag && NodeCnt > ETnodes)
    {
      ElapsedTime (2);
      if (et >= (ResponseTime + ExtraTime))
	if (TCcount < 3)
	  {
	    if (Sdepth < ZDEPTH)
	      {
		ExtraTime += TCleft;
		TCcount++;
	      }
	    else if (Sdepth > 1 && TPrVar1 != PrVar[1])
	      {
		ExtraTime += TCleft;
		TCcount++;
	      }
	    else if (Sdepth > 2 && TPrVar2 != PrVar[2])
	      {
		ExtraTime += TCleft;
		TCcount++;
	      }
	  }
      if (et >= (ResponseTime + ExtraTime))
	flag.timeout = true;

    }
  xside = otherside[side];

  /*
   * check for possible repitition if so call repitition - rpt is repeat
   * count
   */
  if ((ply <= Sdepth + 3) && rpthash[side][hashkey & 0xFF] > 0)
    {
      repetition (rpt);

      /*
       * repeat position >2 don't need to return score it's taken care of
       * above
       */
      if (*rpt == 1 && ply >1) return 0;
    }
  else
    *rpt = 0;

  /* slk is lone king indicator for either side */
  score = evaluate (side, ply, alpha, beta, INCscore, &slk, &InChk);
  /* score > 9000 its a draw or mate */
  if (score > 9000)
    {
      bstline[ply] = 0;
      return (score);
    }
  /* Do we need to add depth because of special conditions */
  /* if in check or pawn threat or in capture sequence search deeper */
  if (depth > 0)		/* we are not at or below our goal depth yet */
    {
      /* Allow opponent a chance to check again */
      if (InChk && depth < 2)
	depth = 2;
      else if (PawnThreat[ply - 1] ||
	       (flag.rcptr && score > alpha && score < beta &&
		ply > 2 && CptrFlag[ply - 1] && CptrFlag[ply - 2]))
	++depth;
    }
  else
    {				/* we are at or below goal depth */
      if (score >= alpha && (InChk || PawnThreat[ply - 1] ||
			     (hung[side] > 1 && ply == Sdepth + 1)))
	depth = 1;

      /*
       * if score  < beta and not 2 levels past goal and have been in check
       * last 2 levels and not on the same square
       */
      else if (score <= beta &&
	       (ply < Sdepth + 4 && ply > 4 &&
		ChkFlag[ply - 2] && ChkFlag[ply - 4] &&
		ChkFlag[ply - 2] != ChkFlag[ply - 4]))
	depth = 1;
    }
  /* try the local transition table if it's there */
#if ttblsz
  if (flag.hash && ply > 1)
    {
      if (ProbeTTable (side, depth, ply, &alpha, &beta, &score) == true)
	{
	  bstline[ply] = PV;
	  bstline[ply + 1] = 0;
	  if (beta == -20000)
	    return (score);
	  if (alpha > beta)
	    return (alpha);
	}
#ifdef HASHFILE
      /* ok try the transition file if its there */
      else if (hashfile && (depth > HashDepth) && (GameCnt < HashMoveLimit)
	       && (ProbeFTable (side, depth, &alpha, &beta, &score) == true))
	{

	  /*
           * if you find something put it in the local table for future
           * reference
           */
	  PutInTTable (side, score, depth, ply, alpha, beta, PV);
	  bstline[ply] = PV;
	  bstline[ply + 1] = 0;
	  if (beta == -20000)
	    return (score);
	  if (alpha > beta)
	    return (alpha);
	}
#endif /* HASHFILE */
    }
#endif /* ttblsz */

  /*
   * if more then DepthBeyond ply past goal depth or at goal depth and
   * score > beta quit - means we are out of the window
   */
  if (ply > Sdepth + DepthBeyond || (depth < 1 && score > beta))
    {
      return (score);
    }

  /*
   * if below first ply and not at goal depth generate all moves else only
   * capture moves
   */
  if (ply > 1)
    if (depth > 0){
      MoveList (side, ply);
      }
    else
      CaptureList (side, ply);

  /* no moves return what we have */

  /*
   * normally a search will continue til past goal and no more capture
   * moves exist
   */
  /* unless it hits Sdepth + DepthBeyond */
  if (TrPnt[ply] == TrPnt[ply + 1])
    {
      return (score);
    }

  /* if not at goal set best = -inf else current score */
  best = ((depth > 0) ? -12000 : score);
  /* if best so far is better than alpha set alpha to best */
  if (best > alpha)
    alpha = best;
  /********************** main loop ************************************************************************/
  /* look at each move until no more or beta cutoff */
  for (pnt = pbst = TrPnt[ply]; pnt < TrPnt[ply + 1] && best <= beta; pnt++)
    {
      /* find the most interesting looking of the remaining moves */
      if (ply > 1)
	pick (pnt, TrPnt[ply + 1] - 1);
      node = &Tree[pnt];
      nxtline[ply + 1] = 0;
#if !defined CHESSTOOL && !defined XBOARD
      /* if at top level */
      if (ply == 1)
	{			/* at the top update search status */
	  if (flag.post)
	    ShowCurrentMove (pnt, node->f, node->t);
	}
#endif
      if (!(node->flags & exact))
	{
	  /* make the move and go deeper */
	  MakeMove (side, node, &tempb, &tempc, &tempsf, &tempst, &INCscore);
	  CptrFlag[ply] = (node->flags & capture);
	  PawnThreat[ply] = (node->flags & pwnthrt);
	  Tscore[ply] = node->score;
	  PV = node->reply;
	  node->score = -search (xside, ply + 1,
				 ((depth > 0) ? depth - 1 : 0),
				 -beta, -alpha,
				 nxtline, &rcnt);
          node->width = (ply%2 == 1)?(TrPnt[ply+2] - TrPnt[ply+1]):0;
	  if (abs (node->score) > 9000)
	    node->flags |= exact;
	  else if (rcnt >= 1)  node->score /= (rcnt+1);
	  /* but why doesnt this detect draws??? */
	  if (rcnt >= 2 || GameCnt - Game50 > 99 || (node->score == 9999 - ply && !ChkFlag[ply]))
	    {
	      node->flags |= (draw | exact);
	      DRAW = "";
	      node->score = ((side == computer) ? contempt : -contempt);
	    }
	  node->reply = nxtline[ply + 1];
	  /* reset to try next move */
	  UnmakeMove (side, node, &tempb, &tempc, &tempsf, &tempst);
	}
      /* if best move so far */
 
      if ((node->score > best && !flag.timeout) || ((node->score == best) && (node->width > bestwidth)))
	{
      /* all things being equal pick the denser part of the tree */
   	  bestwidth = node->width;

	  /*
           * if not at goal depth and better than alpha and not an exact
           * score increment by depth
           */
	  if (depth > 0 && node->score > alpha && !(node->flags & exact))
	    node->score += depth;
	  best = node->score;
	  pbst = pnt;
	  if (best > alpha)
	    {
	      alpha = best;
	    }
	  /* update best line */
	  for (j = ply + 1; nxtline[j] > 0; j++)
	    bstline[j] = nxtline[j];
	  bstline[j] = 0;
	  bstline[ply] = (node->f << 8) | node->t;
	  /* if at the top */
	  if (ply == 1)
	    {

	      /*
	       * if its better than the root score make it the root
	       */
	      if ((best > root->score) || ((best == root->score) && (bestwidth > root->width)))
		{
		  tmp = Tree[pnt];
		  for (j = pnt - 1; j >= 0; j--)
		    Tree[j + 1] = Tree[j];
		  Tree[0] = tmp;
		  pbst = 0;
		}
#if !defined CHESSTOOL && !defined XBOARD
	      if (Sdepth > 2)
		if (best > beta)
		  ShowResults (best, bstline, '+');
		else if (best < alpha)
		  ShowResults (best, bstline, '-');
		else
		  ShowResults (best, bstline, '&');
#endif
	    }
	}
      if (flag.timeout)
	return (-Tscore[ply - 1]);
    }
  /******************************************************************************************/
  node = &Tree[pbst];
  mv = (node->f << 8) | node->t;

  /*
   * we have a move so put it in local table - if it's already there done
   * else if not there or needs to be updated also put it in hashfile
   */
#if ttblsz
  if (flag.hash && *rpt == 0 && best == alpha)
    {
      if (PutInTTable (side, best, depth, ply, alpha, beta, mv)
#ifdef HASHFILE
	  && hashfile && (depth > HashDepth) && (GameCnt < HashMoveLimit))
	{
	  PutInFTable (side, best, depth, alpha, beta, node->f, node->t);
	}
#else
	);
#endif /* HASHFILE */
    }
#endif /* ttblsz */
  if (depth > 0)
    {
      if (node->t != (GameList[GameCnt].gmove & 0xFF))
	if (best <= beta)
	  killr3[ply] = mv;
	else if (mv != killr1[ply])
	  {
	    killr2[ply] = killr1[ply];
	    killr1[ply] = mv;
	  }
      killr0[ply] = ((best > 9000) ? mv : 0);
    }
  return (best);
}




int
castle (short int side, short int kf, short int kt, short int iop)

/* Make or Unmake a castling move. */

{
  register short rf, rt, t0, xside;

  xside = otherside[side];
  if (kt > kf)
    {
      rf = kf + 3;
      rt = kt - 1;
    }
  else
    {
      rf = kf - 4;
      rt = kt + 1;
    }
  if (iop == 0)
    {
      if (kf != kingP[side] ||
	  board[kf] != king ||
	  board[rf] != rook ||
	  color[kf] != side ||
	  color[rf] != side ||
	  Mvboard[kf] != 0 ||
	  Mvboard[rf] != 0 ||
	  color[kt] != neutral ||
	  color[rt] != neutral ||
	  color[kt - 1] != neutral ||
	  SqAtakd (kf, xside) ||
	  SqAtakd (kt, xside) ||
	  SqAtakd (rt, xside))
	return (false);
    }
  else
    {
      if (iop == 1)
	{
	  castld[side] = true;
	  Mvboard[kf]++;
	  Mvboard[rf]++;
	}
      else
	{
	  castld[side] = false;
	  Mvboard[kf]--;
	  Mvboard[rf]--;
	  t0 = kt;
	  kt = kf;
	  kf = t0;
	  t0 = rt;
	  rt = rf;
	  rf = t0;
	}
      board[kt] = king;
      color[rt] = color[kt] = side;
      Pindex[kt] = 0;
      board[kf] = no_piece;
      color[rf] = color[kf] = neutral;
      board[rt] = rook;
      Pindex[rt] = Pindex[rf];
      board[rf] = no_piece;
      PieceList[side][Pindex[kt]] = kt;
      PieceList[side][Pindex[rt]] = rt;
#if ttblsz
      UpdateHashbd (side, king, kf, kt);
      UpdateHashbd (side, rook, rf, rt);
#endif /* ttblsz */
    }
  return (true);
}


void
EnPassant (short int xside, short int f, short int t, short int iop)

/*
 * Make or unmake an en passant move.
 */

{
  register short l;

  l = t + ((t > f) ? -8 : 8);
  if (iop == 1)
    {
      board[l] = no_piece;
      color[l] = neutral;
    }
  else
    {
      board[l] = pawn;
      color[l] = xside;
    }
  InitializeStats ();
  if (iop != 1)
    epsquare = t;
}


void
UpdatePieceList (short int side, short int sq, short int iop)

/*
 * Update the PieceList and Pindex arrays when a piece is captured or when a
 * capture is unmade.
 */

{
  register short i;

  if (iop == 1)
    {
      PieceCnt[side]--;
      for (i = Pindex[sq]; i <= PieceCnt[side]; i++)
	{
	  PieceList[side][i] = PieceList[side][i + 1];
	  Pindex[PieceList[side][i]] = i;
	}
    }
  else
    {
      PieceCnt[side]++;
      PieceList[side][PieceCnt[side]] = sq;
      Pindex[sq] = PieceCnt[side];
    }
}

void
MakeMove (short int side,
	  struct leaf * node,
	  short int *tempb,	/* color of to square */
	  short int *tempc,	/* piece at to square */
	  short int *tempsf,	/* static value of piece on from */
	  short int *tempst,	/* static value of piece on to */
	  short int *INCscore)	/* score increment for pawn structure change */

/*
 * Update Arrays board[], color[], and Pindex[] to reflect the new board
 * position obtained after making the move pointed to by node. Also update
 * miscellaneous stuff that changes when a move is made.
 */

{
  register short f, t, xside, ct, cf;
  register struct GameRec *g;

  xside = otherside[side];
  g = &GameList[++GameCnt];
  f = node->f;
  t = node->t;
  epsquare = -1;
  FROMsquare = f;
  TOsquare = t;
  *INCscore = 0;
  g->Game50 = Game50;
  g->gmove = (f << 8) | t;
  g->flags = node->flags;
  if (node->flags & cstlmask)
    {
      g->piece = no_piece;
      g->color = side;
      (void) castle (side, f, t, 1);
      Game50 = GameCnt;
    }
  else
    {
      if (!(node->flags & capture) && (board[f] != pawn))
	rpthash[side][hashkey & 0xFF]++;
      else
	Game50 = GameCnt;
      *tempsf = svalue[f];
      *tempst = svalue[t];
      g->piece = *tempb = board[t];
      g->color = *tempc = color[t];
      if (*tempc != neutral)
	{
	  UpdatePieceList (*tempc, t, 1);
	  /* if capture decrement pawn count */
	  if (*tempb == pawn)
	    --PawnCnt[*tempc][column (t)];
	  if (board[f] == pawn)
	    {
	      cf = column (f);
	      ct = column (t);
	      /* move count from from to to */
	      --PawnCnt[side][cf];
	      ++PawnCnt[side][ct];

	      /*
	       * calculate increment for pawn structure changes
	       */
	      /* doubled or more - */
	      if (PawnCnt[side][ct] > (1 + PawnCnt[side][cf]))
		*INCscore -= 15;
	      /* went to empty column + */
	      else if (PawnCnt[side][ct] < 1 + PawnCnt[side][cf])
		*INCscore += 15;

	      /*
	       * went to outside col or empty col on one side ????????
	       */
	      else if (ct == 0 || ct == 7 || PawnCnt[side][ct + ct - cf] == 0)
		*INCscore -= 15;
	    }
	  mtl[xside] -= value[*tempb];
	  if (*tempb == pawn)
	    pmtl[xside] -= valueP;
#if ttblsz
	  UpdateHashbd (xside, *tempb, -1, t);
#endif /* ttblsz */
	  *INCscore += *tempst;
	  Mvboard[t]++;
	}
      color[t] = color[f];
      board[t] = board[f];
      svalue[t] = svalue[f];
      Pindex[t] = Pindex[f];
      PieceList[side][Pindex[t]] = t;
      color[f] = neutral;
      board[f] = no_piece;
      if (board[t] == pawn)
	if (t - f == 16)
	  epsquare = f + 8;
	else if (f - t == 16)
	  epsquare = f - 8;
      if (node->flags & promote)
	{
	  board[t] = node->flags & pmask;
	  if (board[t] == queen)
	    HasQueen[side]++;
	  else if (board[t] == rook)
	    HasRook[side]++;
	  else if (board[t] == bishop)
	    HasBishop[side]++;
	  else if (board[t] == knight)
	    HasKnight[side]++;
	  --PawnCnt[side][column (t)];
	  mtl[side] += value[board[t]] - valueP;
	  pmtl[side] -= valueP;
#if ttblsz
	  UpdateHashbd (side, pawn, f, -1);
	  UpdateHashbd (side, board[t], f, -1);
#endif /* ttblsz */
	  *INCscore -= *tempsf;
	}
      if (node->flags & epmask)
	EnPassant (xside, f, t, 1);
      else
#if ttblsz
	UpdateHashbd (side, board[t], f, t);
#else
	 /* NOOP */ ;
#endif /* ttblsz */
      Mvboard[f]++;
    }
}

void
UnmakeMove (short int side,
	    struct leaf * node,
	    short int *tempb,
	    short int *tempc,
	    short int *tempsf,
	    short int *tempst)

/*
 * Take back a move.
 */

{
  register short f, t, xside;

  xside = otherside[side];
  f = node->f;
  t = node->t;
  epsquare = -1;
  Game50 = GameList[GameCnt--].Game50;
  if (node->flags & cstlmask)
    (void) castle (side, f, t, 2);
  else
    {
      color[f] = color[t];
      board[f] = board[t];
      svalue[f] = *tempsf;
      Pindex[f] = Pindex[t];
      PieceList[side][Pindex[f]] = f;
      color[t] = *tempc;
      board[t] = *tempb;
      svalue[t] = *tempst;
      if (node->flags & promote)
	{
	  board[f] = pawn;
	  ++PawnCnt[side][column (t)];
	  mtl[side] += valueP - value[node->flags & pmask];
	  pmtl[side] += valueP;
#if ttblsz
	  UpdateHashbd (side, (short) node->flags & pmask, -1, t);
	  UpdateHashbd (side, pawn, -1, t);
#endif /* ttblsz */
	}
      if (*tempc != neutral)
	{
	  UpdatePieceList (*tempc, t, 2);
	  if (*tempb == pawn)
	    ++PawnCnt[*tempc][column (t)];
	  if (board[f] == pawn)
	    {
	      --PawnCnt[side][column (t)];
	      ++PawnCnt[side][column (f)];
	    }
	  mtl[xside] += value[*tempb];
	  if (*tempb == pawn)
	    pmtl[xside] += valueP;
#if ttblsz
	  UpdateHashbd (xside, *tempb, -1, t);
#endif /* ttblsz */
	  Mvboard[t]--;
	}
      if (node->flags & epmask)
	EnPassant (xside, f, t, 2);
      else
#if ttblsz
	UpdateHashbd (side, board[f], f, t);
#else
	 /* NOOP */ ;
#endif /* ttblsz */
      Mvboard[f]--;
      if (!(node->flags & capture) && (board[f] != pawn))
	rpthash[side][hashkey & 0xFF]--;
    }
}


void
InitializeStats (void)

/*
 * Scan thru the board seeing what's on each square. If a piece is found,
 * update the variables PieceCnt, PawnCnt, Pindex and PieceList. Also
 * determine the material for each side and set the hashkey and hashbd
 * variables to represent the current board position. Array
 * PieceList[side][indx] contains the location of all the pieces of either
 * side. Array Pindex[sq] contains the indx into PieceList for a given
 * square.
 */

{
  register short i, sq;

  epsquare = -1;
  for (i = 0; i < 8; i++)
    PawnCnt[white][i] = PawnCnt[black][i] = 0;
  mtl[white] = mtl[black] = pmtl[white] = pmtl[black] = 0;
  PieceCnt[white] = PieceCnt[black] = 0;
#if ttblsz
  hashbd = hashkey = 0;
#endif /* ttblsz */
  for (sq = 0; sq < 64; sq++)
    if (color[sq] != neutral)
      {
	mtl[color[sq]] += value[board[sq]];
	if (board[sq] == pawn)
	  {
	    pmtl[color[sq]] += valueP;
	    ++PawnCnt[color[sq]][column (sq)];
	  }
	Pindex[sq] = ((board[sq] == king) ? 0 : ++PieceCnt[color[sq]]);

	PieceList[color[sq]][Pindex[sq]] = sq;
#if ttblsz
	hashbd ^= hashcode[color[sq]][board[sq]][sq].bd;
	hashkey ^= hashcode[color[sq]][board[sq]][sq].key;
#endif /* ttblsz */
      }
}
